#This script estimates the four weekly betas

EstimateFourWeekly <- function(EndDate,StartDate,Sample,Firms){
  
Data <- read.csv("Data/FinalDataset_Energy_delisted.csv", header = TRUE, stringsAsFactors = FALSE, strip.white = TRUE) 

####Prepare data for estimation####

# filter time period

#Careful if the .csv has been opened in Excel the date format may have changed

#Data$date <- ymd(Data$date)  #Unhash this and hash the one below if dates don't work

Data$date <- as.Date(Data$date, format = "%d/%m/%Y")

Data <- data.table(Data)

setkey(Data,Code,date)

Data <- Data %>% filter(date <= EndDate, date >= StartDate)

#Filter firms relevant for this sample

Sample <- paste0("Sample_",Sample)

Firms <- filter(Firms,!!as.symbol(Sample) == "Yes") 

Data <- filter(Data,Code %in% Firms$Code)

# Calculate reference days

Data$Weekday <- strftime(Data$date, "%u") %>% as.numeric()  #7 is Sunday, 6/7 can't be trading days

#Remove weekend days

Data <- filter(Data, Weekday != 6 & Weekday != 7)

####Create the four weekly indicator####

dates <- seq.Date(StartDate, EndDate , by = 1)

dates <- as.data.frame(dates)

dates$Weekday <- strftime(dates$dates, "%u") %>% as.numeric()

WeekdayIndicator <- dates$Weekday

#Pick the first weekday 
#Weekends are 6 and 7

Ind <- 1

for(i in 1:length(WeekdayIndicator)){
  if(WeekdayIndicator[i]== 6 | WeekdayIndicator[i]== 7){
    Ind <- Ind + 1
  } else {
    break
  }}

DateVector <- dates$dates

DateVector <- DateVector[Ind:length(DateVector)]

MonthVector <- c(1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28)

MonthVector <- rep(MonthVector, length.out = length(DateVector))

MonthIndicator <- data.frame(date = DateVector, MonthIndicator = MonthVector)

#Join to data

Data <- left_join(Data, MonthIndicator, by = "date")

#Calculate four weekly changes in prices

Data <- Data %>% group_by(Code, MonthIndicator) %>%
  mutate(PX_LAST_FourWeekly = lag(PX_LAST, 1)) %>% as.data.frame()

Data <- Data %>% group_by(Code, MonthIndicator) %>%
  mutate(INDEX_PX_LAST_FourWeekly = lag(INDEX_PX_LAST, 1)) %>% as.data.frame()

#Calculate change in price 

Data <- Data %>%
  mutate(PX_LAST_Fourweekly_change = (PX_LAST - PX_LAST_FourWeekly)/PX_LAST_FourWeekly)

Data <- Data %>%
  mutate(INDEX_PX_LAST_Fourweekly_change = (INDEX_PX_LAST - INDEX_PX_LAST_FourWeekly)/INDEX_PX_LAST_FourWeekly)

# Remove NA

Data <- filter(Data, !is.na(PX_LAST_FourWeekly))
Data <- filter(Data, !is.na(PX_LAST_Fourweekly_change))

#Calculate gearing

Data <- Data %>%
  mutate(GearingDE = NET_DEBT/CUR_MKT_CAP, GearingDV = NET_DEBT/(CUR_MKT_CAP + NET_DEBT))

Data <- Data %>%
  mutate(GearingDEcheck = GearingDV/(1-GearingDV))
Data <- Data %>%
  mutate(GearingDEcheck2 = GearingDE-GearingDEcheck)

####Estimate equity betas####

#Remove any firm where all results in index (i.e. index not identified) are NA

`%notin%` <- Negate(`%in%`)

IndexNA <- Data %>% group_by(Code,Weekday) %>% summarise(test = all(is.na(INDEX_PX_LAST_Fourweekly_change)))
IndexNA <- filter(IndexNA, test == TRUE)
IndexNA <- unique(IndexNA$Code)

Data <- filter(Data, Code  %notin% IndexNA)

#Using a groupwise operation

RegressionResults <- Data %>% group_by(Code, MonthIndicator) %>% do(model = lm(PX_LAST_Fourweekly_change ~ INDEX_PX_LAST_Fourweekly_change, data = .))

#Extract components

RegressionResults <- mutate(RegressionResults, EquityBeta = model[[1]][2], SE = summary(model)$coefficients[4])

####Calculate asset betas####

SummaryValuesAll <- Data %>% group_by(Code, MonthIndicator) %>%
  summarise(AverageGearingDE = mean(GearingDE, na.rm = TRUE), AverageGearingDV = mean(GearingDV, na.rm = TRUE) , AverageTaxRate = mean(IS_STATUTORY_TAX_RATE, na.rm = TRUE))

#Combine Results

RegressionResults <- left_join(RegressionResults, SummaryValuesAll, by = c("Code", "MonthIndicator"))

#Calculate asset betas
#Using Brealey-Meyers

RegressionResults <- mutate(RegressionResults, Assetbeta = EquityBeta/(1+(AverageGearingDE))) %>% ungroup()

#Get full results without model

FullResultsFourWeekly <- RegressionResults %>% select(-model)

FullResultsFourWeekly <- mutate(FullResultsFourWeekly, SE_AssetBeta = SE/(1+(AverageGearingDE))) #Get SE of assetbeta

#Calculate SE

FullResultsFourWeekly <- mutate(FullResultsFourWeekly , SE2 = SE_AssetBeta^2) %>% group_by(Code) %>% mutate(SE2_All = sum(SE2)/n()) %>% 
  ungroup() %>% mutate(SE_All = sqrt(SE2_All))

#Average across all four weekly days 

ResultsAvgFourWeekly <- FullResultsFourWeekly %>% group_by(Code) %>% summarise(Assetbeta = mean(Assetbeta), gearingDV = mean(AverageGearingDV), SE_AssetBeta = mean(SE_All))

####Estimate standard errors#####

#Based on 2016 Commmission spreadsheet tab: Std error Ba

SEEstimate <- ResultsAvgFourWeekly %>% mutate(Variance = SE_AssetBeta ^ 2)

SEEstimate <- SEEstimate %>% summarise(Avg_Assetbeta = mean(Assetbeta), N = n(), v = (N/(N-1) * var(Assetbeta)), SigmaE = mean(Variance), 
                                       Rho = 0.2, Variance = v * ((N + 1)/N) - (SigmaE * (1-2*Rho)),
                                       StandardError = sqrt(Variance))

####Writeout####

write.csv(ResultsAvgFourWeekly, paste0("Energy analysis - Vector/Results/StartDate-",StartDate,"EndDate-",EndDate,"-FourWeekly-" ,"Results.csv"), row.names = FALSE)

write.csv(SEEstimate, paste0("Energy analysis - Vector/Results/StartDate-",StartDate,"EndDate-",EndDate,"-FourWeekly-" ,"StandardErrorResults.csv"), row.names = FALSE)

}
