#This script estimates the daily betas

####Setup####

library(tseries)

####Read in inputs####

EstimateDaily <- function(EndDate,StartDate,Sample,Firms,Covid_file,EventD,Covid_date){

# AS changed data read to provided two data files
Data <- read.csv("Data/FinalDataset_Energy_delisted.csv", header = TRUE, stringsAsFactors = FALSE, strip.white = TRUE)

# AS these assignments are not necessary but shouldn't affect anything  

EndDate <- EndDate

StartDate <- StartDate

Firms <- Firms

Sample <- Sample

Covid_file <- Covid_file

Event <- EventD

Covid_date <- ymd(Covid_date)

####Prepare data for estimation####

# filter time periods

#Careful if the .csv has been opened in Excel the date format may have changed

Data$date <- as.Date(Data$date, format = "%d/%m/%Y") #If an error occurs, hash and use line below

#Data$date <- ymd(Data$date)

Data <- data.table(Data)

setkey(Data,Code,date)

Data <- Data %>% arrange(Code, date) %>%    
  filter(date <= EndDate, date >= StartDate)

#Filter firms relevant for this sample

Sample <- paste0("Sample_",Sample)

Firms <- filter(Firms,!!as.symbol(Sample) == "Yes") 
Data <- filter(Data,Code %in% Firms$Code)

#Replace index with NZX50 for replicating TBD Advisory results
# Data_NZX50 <- read.csv("NZX50.csv")
# Data_NZX50$date <- as.Date(Data_NZX50$date, format = "%d/%m/%Y")
# Data <- left_join(Data, Data_NZX50, by = "date")
# Data$INDEX_PX_LAST <- NULL
# setnames(Data,"NZX50","INDEX_PX_LAST")

# Calculate reference days

Data$Weekday <- strftime(Data$date, "%u") %>% as.numeric()  #7 is Sunday, 6/7 can't be trading days

#Remove weekend days

Data <- filter(Data, Weekday != 6 & Weekday != 7)

#Calculate Daily changes in prices

Data <- Data %>% group_by(Code) %>%
  mutate(PX_LAST_Daily = lag(PX_LAST, 1)) %>% as.data.frame()

Data <- Data %>% group_by(Code) %>%
  mutate(INDEX_PX_LAST_Daily = lag(INDEX_PX_LAST, 1)) %>% as.data.frame()

#Calculate % change in price 

Data <- Data %>%
  mutate(PX_LAST_Daily_change = (PX_LAST - PX_LAST_Daily)/PX_LAST_Daily)

Data <- Data %>%
  mutate(INDEX_PX_LAST_Daily_change = (INDEX_PX_LAST - INDEX_PX_LAST_Daily)/INDEX_PX_LAST_Daily)

# Remove NA

Data <- filter(Data, !is.na(PX_LAST_Daily))
Data <- filter(Data, !is.na(PX_LAST_Daily_change))

#Calculate gearing

Data <- Data %>%
  mutate(GearingDE = NET_DEBT/CUR_MKT_CAP, GearingDV = NET_DEBT/(CUR_MKT_CAP + NET_DEBT))

Data <- Data %>%
  mutate(GearingDEcheck = GearingDV/(1-GearingDV))
Data <- Data %>%
  mutate(GearingDEcheck2 = GearingDE-GearingDEcheck)

####Estimate equity betas####

#Error handling

#Remove any firm where all results in index (i.e. index not identified) are NA

`%notin%` <- Negate(`%in%`)

IndexNA <- Data %>% group_by(Code) %>% summarise(test = all(is.na(INDEX_PX_LAST_Daily_change)))
IndexNA <- filter(IndexNA, test == TRUE)
IndexNA <- unique(IndexNA$Code)

Data <- filter(Data, Code  %notin% IndexNA)

#Apply weights to replicate the Flint method

Data$weight <- ifelse(Data$date < "2020-02-01", Event,
                      ifelse(Data$date > as.Date(Covid_date), Event,1))

Data$WIndex <- Data$INDEX_PX_LAST_Daily_change * Data$weight
Data$WPrice <- Data$PX_LAST_Daily_change * Data$weight

#Use a groupwise operation to estimate betas

RegressionResults <- Data %>% group_by(Code) %>% do(model = lm(WPrice ~ WIndex, data = .))

#Extract components

RegressionResults <- mutate(RegressionResults, EquityBeta = model[[1]][2], SE = summary(model)$coefficients[4])

####Calculate asset betas####

#Calculate average gearing

SummaryValuesAll <- Data %>% group_by(Code) %>%
  summarise(
    AverageGearingDE = mean(GearingDE, na.rm = TRUE), 
    AverageGearingDV = mean(GearingDV, na.rm = TRUE) ,
    AverageTaxRate = mean(IS_STATUTORY_TAX_RATE, na.rm = TRUE)
  )

#Combine Results

RegressionResults <- left_join(RegressionResults, SummaryValuesAll, by = c("Code"))

#Calculate asset betas
#Using Brealey-Meyers

RegressionResults <- mutate(RegressionResults, Assetbeta = EquityBeta/(1+(AverageGearingDE)), SE_AssetBeta = SE/(1+(AverageGearingDE)) ) %>% ungroup()

#Get full results without model

FullResults <- RegressionResults %>% select(-model)

####Estimate standard errors#####

#Based on 2016 Commmission spreadsheet tab: Std error Ba

SEEstimate <- FullResults %>% mutate(Variance = SE_AssetBeta ^ 2)

SEEstimate <- SEEstimate %>% summarise(Avg_Assetbeta = mean(Assetbeta),N = n(), v = (N/(N-1) * var(Assetbeta)), SigmaE = mean(Variance), 
                                       Rho = 0.2, Variance = v * ((N + 1)/N) - (SigmaE * (1-2*Rho)),
                                       StandardError = sqrt(Variance))


####Writeout####

write.csv(FullResults, paste0("Flint analysis - Energy/Results/","Daily",Covid_file), row.names = FALSE)


}
