#This function estimates the weekly betas

####Setup####

#library(stringr)
#library(Rblpapi)
#library(dplyr)
#library(zoo)
#library(tidyr)

####Read in inputs####

EstimateWeekly <- function(EndDate,StartDate,Sample,Firms,Covid_file,EventW,Covid_date){

  # AS changed data read to provided two data files
  Data <- read.csv("Data/FinalDataset_Energy_delisted.csv", header = TRUE, stringsAsFactors = FALSE, strip.white = TRUE)
  
  EndDate <- EndDate
  
  StartDate <- StartDate
  
  Firms <- Firms
  
  Sample <- Sample
  
  Covid_file <- Covid_file
  
  Event <- EventW
  
  Covid_date <- ymd(Covid_date)
  
  
  ####Prepare data for estimation####
  
  # filter time periods
  
  #Careful if the .csv has been opened in Excel the date format may have changed
  
  Data$date <- as.Date(Data$date, format = "%d/%m/%Y") 
  
  #Data$date <- ymd(Data$date)
  
  Data <- data.table(Data)
  
  setkey(Data,Code,date)
  
  Data <- Data %>% filter(date <= EndDate, date >= StartDate)
  
  #Filter firms relevant for this sample
  
  Sample <- paste0("Sample_",Sample)
  
  Firms <- filter(Firms,!!as.symbol(Sample) == "Yes") #Odd construction here, but this simply selects the appropriate column programmatically
  
  Data <- filter(Data,Code %in% Firms$Code)
  
  #Replace index with NZX50 for replicating TBD Advisory results
  # Data_NZX50 <- read.csv("Data/NZX50.csv")
  # Data_NZX50$date <- as.Date(Data_NZX50$date, format = "%d/%m/%Y")
  # Data <- left_join(Data, Data_NZX50, by = "date")
  # Data$INDEX_PX_LAST <- NULL
  # setnames(Data,"NZX50","INDEX_PX_LAST")
  # Data <- filter(Data, !is.na(INDEX_PX_LAST))
  
  # Calculate reference days
  
  Data$Weekday <- strftime(Data$date, "%u") %>% as.numeric()  #7 is Sunday, 6/7 can't be trading days
  
  #Remove weekend days
  
  Data <- filter(Data, Weekday != 6 & Weekday != 7)
  
  #Calculate weekly changes in prices
  
  Data <- Data %>% group_by(Code, Weekday) %>%
    mutate(PX_LAST_Weekly = lag(PX_LAST, 1)) %>% as.data.frame()
  
  Data <- Data %>% group_by(Code, Weekday) %>%
    mutate(INDEX_PX_LAST_Weekly = lag(INDEX_PX_LAST, 1)) %>% as.data.frame()
  
  #Calculate change in price 
  
  Data <- Data %>%
    mutate(PX_LAST_weekly_change = (PX_LAST - PX_LAST_Weekly)/PX_LAST_Weekly)
  
  Data <- Data %>%
    mutate(INDEX_PX_LAST_weekly_change = (INDEX_PX_LAST - INDEX_PX_LAST_Weekly)/INDEX_PX_LAST_Weekly)
  
  # Remove NA
  
  Data <- filter(Data, !is.na(PX_LAST_Weekly))
  Data <- filter(Data, !is.na(PX_LAST_weekly_change))
  Data <- filter(Data, !is.na(INDEX_PX_LAST_weekly_change)) #GW added to replicate TBD Advisory
  
  #Calculate gearing
  
  Data <- Data %>%
    mutate(GearingDE = NET_DEBT/CUR_MKT_CAP, GearingDV = NET_DEBT/(CUR_MKT_CAP + NET_DEBT))
  
  Data <- Data %>%
    mutate(GearingDEcheck = GearingDV/(1-GearingDV))
  Data <- Data %>%
    mutate(GearingDEcheck2 = GearingDE-GearingDEcheck)
  
  ####Estimate equity betas####
  
  #Error handling
  
  #Remove any firm where all results in index (i.e. index not identified) are NA
  
  `%notin%` <- Negate(`%in%`)
  
  IndexNA <- Data %>% group_by(Code) %>% summarise(test = all(is.na(INDEX_PX_LAST_weekly_change)))
  IndexNA <- filter(IndexNA, test == TRUE)
  IndexNA <- unique(IndexNA$Code)
  
  Data <- filter(Data, Code  %notin% IndexNA)
  
  #Apply weights to replicate the Flint method
  
  Data$weight <- ifelse(Data$date < "2020-02-01", Event,
                        ifelse(Data$date > as.Date(Covid_date), Event,1))
  
  Data$WIndex <- Data$INDEX_PX_LAST_weekly_change * Data$weight
  Data$WPrice <- Data$PX_LAST_weekly_change * Data$weight
  
  #Using a groupwise operation (individual day of week regressions)
  
  RegressionResults <- Data %>% group_by(Code, Weekday) %>% do(model = lm(WPrice ~ WIndex, data = .))
  
  #Using a groupwise operation (pooled regression)
  
  RegressionResultsPooled <- Data %>% group_by(Code) %>% do(model = lm(WPrice ~ WIndex, data = .))
  
  #Extract components
  
  RegressionResults <- mutate(RegressionResults, EquityBeta = model[[1]][2], SE = summary(model)$coefficients[4])
  RegressionResultsPooled <- mutate(RegressionResultsPooled, EquityBeta = model[[1]][2], SE = summary(model)$coefficients[4])
  
  ####Calculate asset betas####
  
  #Calculate average gearing
  
  SummaryValuesAll <- Data %>% group_by(Code, Weekday) %>%
    summarise(AverageGearingDE = mean(GearingDE, na.rm = TRUE), AverageGearingDV = mean(GearingDV, na.rm = TRUE) , AverageTaxRate = mean(IS_STATUTORY_TAX_RATE, na.rm = TRUE))
  
  #Combine Results
  
  RegressionResults <- left_join(RegressionResults, SummaryValuesAll, by = c("Code", "Weekday"))
  
  #Calcualte asset betas
  #Using Brealey-Meyers
  
  RegressionResults <- mutate(RegressionResults, Assetbeta = EquityBeta/(1+(AverageGearingDE))) %>% ungroup()
  
  #Get full results without model
  
  FullResultsWeekly <- RegressionResults %>% select(-model)
  
  FullResultsWeekly <- mutate(FullResultsWeekly, SE_AssetBeta = SE/(1+(AverageGearingDE))) #Get SE of assetbeta
  
  #Calculate SE
  
  FullResultsWeekly <- mutate(FullResultsWeekly , SE2 = SE_AssetBeta^2) %>% group_by(Code) %>% mutate(SE2_All = sum(SE2)/n()) %>% 
    ungroup() %>% mutate(SE_All = sqrt(SE2_All))
  
  #Average across all weekdays
  
  ResultsAvgWeekdays <- FullResultsWeekly %>% group_by(Code) %>% summarise(Assetbeta = mean(Assetbeta), gearingDV = mean(AverageGearingDV), SE_AssetBeta = mean(SE_All))
  
  ####Estimate standard errors#####
  
  #Based on 2016 Commmission spreadsheet tab: Std error Ba
  
  SEEstimate <- ResultsAvgWeekdays %>% mutate(Variance = SE_AssetBeta ^ 2)
  
  SEEstimate <- SEEstimate %>% summarise(Avg_Assetbeta = mean(Assetbeta), N = n(), v = (N/(N-1) * var(Assetbeta)), SigmaE = mean(Variance), 
                                         Rho = 0.2, Variance = v * ((N + 1)/N) - (SigmaE * (1-2*Rho)),
                                         StandardError = sqrt(Variance))
  
  ####Writeout####
  
  write.csv(ResultsAvgWeekdays, paste0("Flint analysis - Vector/Results/","Weekly",Covid_file), row.names = FALSE)
  
  
}
